<p align="center">
  <a href="https://github.com/assemble/assemble">
    <img height="250" width="250" src="https://raw.githubusercontent.com/assemble/assemble/master/docs/logo.png">
  </a>
</p>

# {%= name %} 

{%= badge("fury") %} {%= badge('downloads') %} {%= ifExists(["test", "test.js"], badge('travis')) %} {%= badge('gitter') %} 

Welcome to Assemble v{%= version %}! 

Assemble is a command line tool and developer framework for rapid prototyping, creating themes, scaffolds, boilerplates, e-books, UI components, API documentation, blogs, building websites / static site generator, alternative to jekyll for gh-pages and more!

_(Note that the current website, assemble.io, is for [grunt-assemble][]. Thanks for your patience while we work on updating the site with documentation for the latest assemble)_.

## Who uses assemble?

Assemble is used by thousands of projects, including:

- [Airbus Group](http://www.airbusgroup.com/int/en.html)
- [hillaryclinton.com](https://www.hillaryclinton.com/)
- [Diebold](http://www.diebold.com/)
- [Transformicons](http://www.transformicons.com/)
- [Barrel](https://www.barrelny.com/)
- [yesware](https://www.yesware.com/)
- [Amaze](https://www.amaze.com/)
- [sennheiser](http://sennheiser-d1.com/)
- [perf.rocks](http://perf.rocks/)
- [Milano JS](http://milanojs.com/)
- [AKQA](http://www.akqa.com/)
- [huxtaburger](http://www.huxtaburger.com.au/)
- [Typeplate](http://typeplate.com/)
- [Angular Basics](http://www.angularjsbook.com/)

## Features

Assemble can be used to create:

* [blogs](examples/blog)
* [build system](examples/build-tool)
* documentation
* e-books
* front-end UI
* [generate boilerplates](examples/boilerplates)
* [generate targets](examples/targets)
* landing pages
* [project generator](examples/generator)
* [project scaffolds](examples/scaffold)
* rapid prototyping
* static site generator
* styleguides
* themes

**Project lifecycle**

As a standalone library, assemble can be used for creating, building and maintaining entire web projects. But it's even more powerful when used longside the following libraries:

- [generate][]: scaffold out new projects
- [assemble][]: build web projects (<= you are here)
- [verb][]: create project documention
- [update][]: maintain projects

Assemble also has a [grunt plugin][grunt-assemble] and can be used alongside [gulp][].

**Features**

* Full support for [gulp][] plugins
* Support for [base][] plugins
* Assemble templates are [vinyl][] files
* render templates with any [template engine](#engine), including [nunjucks][assemble-nunjucks], [handlebars][engine-handlebars], [lodash][engine-lodash] and any consolidate engine!
* [helpers](#helpers): support for sync and async
* [templates collections](#collections) support
* partials and includes
* layouts
* pages
* custom template "types"
* pagination
* [permalinks][assemble-permalinks]
* [middleware](#middleware) can be used to tranform files at any stage in the render cycle
* pagination
* Much more!


## Table of contents

<!-- toc -->

## Getting started

### Install assemble

To use assemble's CLI, you will first need to install it globally using [npm](https://www.npmjs.com):

```sh
$ npm --global install assemble
```

This adds the `assemble` command to your system path, allowing it to be run from any directory.


### Example usage

#### Rendering templates

Render a template with with [handlebars][]:

```js
var assemble = require('{%= name %}');
var app = assemble();
var locals = {title: 'Home!'};

// add a "page" 
app.page('home.hbs', {content: 'This is {{title}}'});

// render it!
app.render('home.hbs', locals, function(err, view) {
  if (err) throw err;
  console.log(view.content);
  //=> 'This is Home!'
});
```

#### Running tasks

Create an `assemblefile.js` and add tasks to run:

```js
var assemble = require('assemble');
var htmlmin = require('gulp-htmlmin');
var app = assemble();

app.page('a.hbs', {content: '...'});
app.page('b.hbs', {content: '...'});
app.page('c.hbs', {content: '...'});

app.task('default', function() {
  return app.toStream('pages') //<= push "pages" collection into stream
    .pipe(app.renderFile()) //<= render pages with default engine (hbs)
    .pipe(htmlmin()) //<= gulp plugin for minifying html
    .pipe(app.dest('site')); //<= write files to the "./site" directory
});

// expose your instance of assemble to assemble's CLI
module.exports = app;
```

## CLI

Run assemble from the command line.

```sh
$ assemble <tasks> [options]
```

### Running tasks

Specify one or more space-separated tasks to run.

**Examples**

Run task `foo`

```sh
$ assemble foo
```

Run tasks `foo` and `bar`

```sh
$ assemble foo bar
```

### Specifying options

Non-task options are prefixed with `--`.

**Examples**

Set the `--cwd` to run an assemblefile.js in a different directory:

```sh
$ assemble --cwd=docs
```

Emit views as they're loaded and log them to `stderr`:

```sh
$ assemble --emit=view
```

See more [command line options]()

### Object expansion

Object-paths may be specified using dot-notation for **either the key or value** in a command line argument.

Additionally, assemble uses [expand-object][] (and some custom parsing) to make it easier to pass non-trivial options and commands via command line. So all of the following formats are possible.

**Examples**

Boolean values:

```sh
$ assemble --foo 
# { foo: true }
```

Key-value pairs:

```sh
$ assemble --foo=bar
# { foo: 'bar' }
```

Nested booleans:

```sh
$ assemble --option=foo 
# {options: { foo: true }}
```

Nested key-value pairs:

```sh
$ assemble --option=foo:bar
# {options: { foo: 'bar' }}
```

Deeply nested key-value pairs:

```sh
$ assemble --option=foo.bar.baz:qux
# {options: foo: { bar: { baz: 'qux' }}}}
```

Or on the left-side of the `=`:

```sh
$ assemble --option.foo.bar.baz=qux
# {options: foo: { bar: { baz: 'qux' }}}}
```

## Command line options

### cwd

Change the `cwd` for the `assemblefile.js` to run, optionally specifying any tasks to run:

```sh
$ assemble <tasks> --cwd [directory]
```

**Example**

To run the `scaffolds` example in the `examples/` directory, you would enter:

```sh
$ assemble --cwd examples/scaffolds
```

If successful, in the command line, you should see something like this:

<img width="527" alt="screen shot 2016-01-09 at 1 35 52 pm" src="https://cloud.githubusercontent.com/assets/383994/12217685/0a14294e-b6d6-11e5-9e06-dc4738f0e53a.png">

### file

Specify the name of the config file for assemble's CLI to run, the default is `assemblefile.js`.

**Example**

```sh
$ assemble --file assemblefile.dev.js
```

## API
{%= apidocs("index.js") %}

### Templates API

Assemble exposes the entire API from the [templates][] library for working with templates and template collections.

#### .create

Create a template collection.

```js
app.create('includes');
```

Add views to the collection.


***

### File System API

Assemble offers the following low-level methods for working with the file system:

- [src](#src)
- [symlink](#symlink)
- [dest](#dest)
- [copy](#copy)

Assemble has first-class support for [vinyl-fs][], so any [gulp][] plugin can be used in your assemble pipeline.

#### .src

Create a [vinyl][] stream. Takes glob patterns or filepaths to the source files to read.

**Params**

* `glob` **{String|Array}**: Glob patterns or file paths to source files.
* `options` **{Object}**: Options or locals to merge into the context and/or pass to `src` plugins

**Example**

```js
app.src('src/*.hbs');

// define `src` options
app.src('src/*.hbs', { layout: 'default' });
```

#### .dest

Specify a destination for processed files.

**Params**

* `dest` **{String|Function}**: File path or rename function.
* `options` **{Object}**: Options and locals to pass to `dest` plugins

**Example**

```js
app.dest('dist/');
```

#### .copy

Copy files with the given glob `patterns` to the specified `dest`.

**Params**

* `patterns` **{String|Array}**: Glob patterns of files to copy.
* `dest` **{String|Function}**: Desination directory.
* `returns` **{Stream}**: Stream, to continue processing if necessary.

**Example**

```js
app.task('assets', function() {
  // return, to let assemble know when the task has completed
  return app.copy('assets/**', 'dist/');
});
```

#### .symlink

Same as `.src` but takes glob patterns or filepaths for the symlinks to read.

**Params**

* `glob` **{String|Array}**: Glob patterns or file paths 

**Example**

```js
app.symlink('src/*.hbs');
```

***

### Task API

Assemble has the following methods for running tasks and controlling workflows:

- [task](#task)  
- [build](#build)  
- [watch](#watch)  

#### .task

Define a task to be run when the task is called.

**Params**

* `name` **{String}**: Task name
* `fn` **{Function}**: function that is called when the task is run.

**Example**

```js
app.task('default', function() {
  app.src('templates/*.hbs')
    .pipe(app.dest('site/'));
});
```

#### .build

Run one or more tasks.

**Params**

* `tasks` **{Array|String}**: Task name or array of task names.
* `cb` **{Function}**: callback function that exposes `err`

**Example**

```js
app.build(['foo', 'bar'], function(err) {
  if (err) throw err;
  console.log('done!');
});
```

#### .watch

Watch files, run one or more tasks when a watched file changes.

**Params**

* `glob` **{String|Array}**: Filepaths or glob patterns.
* `tasks` **{Array}**: Task(s) to watch.

**Example**

```js
app.task('watch', function() {
  app.watch('docs/*.md', ['docs']);
});
```

## Plugins

### Discovering plugins

Plugins from any applications built on [base][] should work with Assemble and can be used in your `assemblefile.js`:

* [base][base-plugin]: find base plugins on npm using the `baseplugin` keyword
* [assemble][assemble-plugin]: find assemble plugins on npm using the `assembleplugin` keyword
* [generate][generate-plugin]: find generate plugins on npm using the `generateplugin` keyword
* [templates][templates-plugin]: find templates plugins on npm using the `templatesplugin` keyword
* [update][update-plugin]: find update plugins on npm using the `updateplugin` keyword
* [verb][verb-plugin]: find verb plugins on npm using the `verbplugin` keyword

### Authoring plugins

Visit the [plugin documentation](docs/plugins.md) guide to learn how to use, author and publish plugins.

## Learning

### Help

**Get in touch!**

Have questions, suggestions, or want to discuss assemble? Join the conversation on [gitter](http://gitter.im/assemble/assemble) or give us a shout on [twitter](https://twitter.com/assemblejs). The assemble team and community are always happy to help! 

### More information

* [Documentation](docs)
* [API documentation](docs/api)
* [Generaters maintained by the core team](https://github.com/generate)

### FAQ

**Website is outdated and being refactored!**

Assemble's website, assemble.io, only has information related to [gulp-assemble][]. We're working hard to update the site with information about the latest release. 

In the meantime, you might find the [WIP docs](docs/src/content) useful. The [unit tests](test/) are also great examples! 

**Is the assemble website up-to-date?**

No, as mentioned above, it's completely out-of-date. If you're using [grunt-assemble][], some of the documentation at assemble.io might still be useful. If you're using assemble v0.6.0 and higher, the documentation is probably wrong in almost every way. 

We're actively (daily) working on a refactor and it's a very high priority. 

**What's the difference between [assemble-core][] and assemble?**

Assemble adds a CLI, a few built-in view collections: `pages`, `layouts`, and `partials`, middleware for parsing front-matter, and a few other basic defaults that we've found many users expect. If you'd prefer different defaults, [assemble-core][] is a great starting point. 

If you want something that handles templates, rendering, engines, helpers, collections, etc. but you don't need to run tasks or work with the file system, then consider using [templates][] instead of assemble-core.

**I use gulp, why is it recommended to use assemble directly, instead of running assemble with gulp?**

You can run gulp plugins with assemble, but it won't always work the other way around. This is because, as a build system, assemble does things that gulp doesn't do, like handle middleware. 

For example, assemble's `.src` and `.dest` methods have built-in `.onStream`, `.preWrite`, and `.postWrite` middleware handlers. If you still wish to use gulp and your build cycle includes middleware that requires these handlers, you can use the [assemble-handle][] plugin with gulp to ensure that the handlers are still called as needed.

This is a long way of saying, you can find ways to make gulp work, but you would just be adding an extra dependency to your project to do things that assemble already does.

**What is the relationship between gulp and assemble?**

Please read our [gulp FAQ](docs/src/subjects/gulp-faq.md) for more information.

## About

### Community

Are you using assemble in your project? Have you published an assemble project and want to share your project with the world? 

Here are some suggestions!

* If you get like Assemble and want to tweet about it, please feel free to mention `@assemble` or use the `#assemble` hashtag
* Tell us about [your assemble project](https://github.com/assemble/assemble/issues/300)
* Show your love by starring [Assemble][] and `{%= name %}`
* Get implementation help on [StackOverflow](http://stackoverflow.com/questions/tagged/assemble) (please use the `assemble` tag in questions)
* **Gitter** Discuss Assemble with us on [Gitter](https://gitter.im/assemble/assemble)
* If you publish an assemble plugin, thank you! To make your project as discoverable as possible, please add the keyword `assembleplugin` to package.json.

**Contributing**

Please read our [contributing guide](.github/contributing.md) if you'd like to learn more about contributing to this project.

[docs]: https://github.com/assemble/assemble/blob/master/docs/

### Related projects

You might also be interested in these projects from [@doowb](https://github.com/doowb) and [@jonschlinkert](https://github.com/jonschlinkert):

{%= related(verb.related.list) %}

### Similar projects

If assemble doesn't do what you need, there are some other great open source projects you might be interested in, created by our friends on GitHub (in alphabetical order):

**Static site generators**

- [docpad](https://github.com/docpad)
- [metalsmith](https://github.com/metalsmith)
- [punch](https://github.com/laktek/punch)
- [wintersmith](https://github.com/jnordberg/wintersmith)

**Blog frameworks**

- [hexojs](https://github.com/hexojs)
- [ghost](https://github.com/TryGhost/Ghost)


### Release history
{%= docs("changelog.md") %}

### Contributing
{%= include("contributing") %}

If Assemble doesn't do what you need, [please let us know]({%= issue({
  title: "request: <title here>",
  repo: "assemble",
  owner: "assemble"
}) %})

### Authors

**Jon Schlinkert**

* [github/jonschlinkert](https://github.com/jonschlinkert)
* [twitter/jonschlinkert](http://twitter.com/jonschlinkert)

**Brian Woodward**

* [github/doowb](https://github.com/doowb)
* [twitter/doowb](http://twitter.com/doowb)

### License
{%= copyright() %}
{%= license %}

***

{%= include("footer") %}


[grunt-assemble]: [assemble]{grunt-assemble}
[assemble-nunjucks]: [assemble]{assemble-nunjucks}
[engine-handlebars]: [github]{jonschlinkert/engine-handlebars}
[engine-lodash]: [github]{jonschlinkert/engine-lodash}
[engine-base]: [github]{jonschlinkert/engine-base}
[assemble]: [github]{assemble}
[github]: https://github.com